import router from './router'
import store from './store'
import { Message } from 'element-ui'
import NProgress from 'nprogress' // progress bar
import 'nprogress/nprogress.css' // progress bar style
import { getToken } from '@/utils/auth' // get token from cookie
import getPageTitle from '@/utils/get-page-title'
import Layout from '@/layout'
import { setTree, listToTree } from '@/utils/validate'
import * as _ from 'lodash'

NProgress.configure({ showSpinner: false }) // NProgress Configuration

const whiteList = ['/login'] // no redirect whitelist

function setStaticRoutes (menus, staticRoutes) {
  staticRoutes.forEach(route => {
    const routeIndex = menus.findIndex(
      permissionRoute => permissionRoute.name === route.name
    )
    if (routeIndex > -1) {
      menus[routeIndex] = {
        ...menus[routeIndex],
        component: route.component,
        children: route.children,
      }
    }
  })
}

router.beforeEach(async (to, from, next) => {
  // start progress bar
  NProgress.start()

  // set page title
  document.title = getPageTitle(to.meta.title)

  // determine whether the user has logged in
  const hasToken = getToken()
  if (hasToken) {
    if (to.path === '/login') {
      // if is logged in, redirect to the home page
      next({ path: '/' })
      NProgress.done()
    } else {
      const hasRoles = store.getters.roles && store.getters.roles.length > 0
      if (hasRoles) {
        next()
      } else {
        try {
          const treeData = await store.dispatch('user/getMenuInfo')
          const nd = treeData.filter(item => item.pageid)
          store.dispatch('auth/changeAuthArrs', nd)
          // const typeData = setType(treeData, 'up_auth_id', 'auth_id')
          const menusArray = parseMenuItem(nd)
          const treeMenu = listToTree(menusArray, 'superiorid', 'pageid')
          const dataMenu = _.unionBy(treeMenu, 'pageid')
          const menus = parseMenu(dataMenu)
          store.dispatch('permission/setRoutes', menus)
          router.addRoutes(menus)
          next({ ...to, replace: true })
        } catch (error) {
          // remove token and go to login page to re-login
          await store.dispatch('user/resetToken')
          Message.error(error || 'Has Error')
          next(`/login?redirect=${to.path}`)
          NProgress.done()
        }
      }
    }
  } else {
    /* has no token*/

    if (whiteList.indexOf(to.path) !== -1) {
      // in the free login whitelist, go directly
      next()
    } else {
      // other pages that do not have permission to access are redirected to the login page.
      next(`/login?redirect=${to.path}`)
      NProgress.done()
    }
  }
})

function parseMenuItem (data) {
  const menus = []
  data.map(item => {
    if (item.opentype == 1) {
      menus.push({
        auth_id: item.pageid,
        up_auth_id: item.superiorid,
        path: item.router,
        name: item.pagedescribe,
        meta: {
          title: item.pagename,
          elSvgIcon: item.icon ?? 'Fold',
          show_index: item.defaultindex,
          qid: item.serviceid,
        },
        component: Layout,
        hidden: !item.is_show,
      })
    } else if (item.opentype == 2) {
      menus.push({
        auth_id: item.pageid,
        up_auth_id: item.superiorid,
        path: item.router,
        name: item.pagedescribe,
        meta: {
          title: item.pagename,
          elSvgIcon: item.icon ?? 'Fold',
          show_index: item.defaultindex,
          qid: item.queryTemplateID,
          auth_id: item.pageid,
          up_auth_id: item.superiorid,
          pagecode: item.pagecode,
          superiorcode: item.superiorcode,
          sort: item.sort,
          tagetpageid: item.tagetpageid
        },
        component: resolve => require(['@/views' + item.filepath], resolve),
        // component: () => import('./views/table/index.vue'),
        hidden: !item.is_show,
      })
    }
  })

  return menus
}

function setType (arr, parentKey, key) {
  const datas = []
  for (let i = 0; i < arr.length; i++) {
    for (let j = 0; j < arr.length; j++) {
      if (arr[i][key] == arr[j][parentKey]) {
        if (arr[j].auth_type == 3) {
          datas.push(arr[j])
          arr[i].other = datas
        }
      }
    }
  }
  return arr
}

function parseMenu (arr) {
  const menus = arr
  const newMenus = menus.flat()
  const allMenus = newMenus.length
    ? newMenus.sort((a, b) => a.meta.show_index - b.meta.show_index)
    : []
  // allMenus.map(item => {
  //   item.children = (item.children && item.children.length) ? item.children.sort((a, b) => a.meta.show_index - b.meta.show_index) : item.children
  // })
  allMenus.push({
    path: '/:pathMatch(.*)',
    redirect: '/404',
    hidden: true,
  })
  return allMenus
}

router.afterEach(() => {
  // finish progress bar
  NProgress.done()
})
